/*	XOP1.c -- A sample Igor external operation

	XOP1 is a very simple Igor XOP (external operation) designed to show you how to get
	started writing XOPs.
	
	The syntax for XOP1 is:
		XOP1 
	
*/

#include "XOPStandardHeaders.h"			// Include ANSI headers, Mac headers, IgorXOP.h, XOP.h and XOPSupport.h
#include "XOP1.h"
#include "buffered_streambuf.hpp"
#include "MyManip.h"


#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TKey.h"
#include <string>
#include <vector>
#include <ostream>
#include "boost/multi_array.hpp"
#include "Wave.hpp"
#include "dataFolderUtilities.h"
#include "fileUtilities.h"
#include "FilePath.h"

#include "load.h"

using namespace std;

static  buffered_streambuf mybuf;
static ostream history(&mybuf);

static void 
loadRecursively(TDirectory &f, TIter & it){
  TKey * key(0);
  while ((key = (TKey*) it())){
    string className=key->GetClassName();
    string name=key->GetName();
    if (className.find("TDirectory") not_eq string::npos){
      TDirectory * thisDirectory=(TDirectory *) key->ReadObj();
      history<<"creating directory for a "<<className<<": "<<name<<inl;
      TIter nextInDir(thisDirectory->GetListOfKeys());
      useOrCreateFolder(name);
      loadRecursively(*thisDirectory,nextInDir);
      cd("..");
    }else {
      if (not loadFromTFile(f, className,name)) history<<name<<" of class "<<className<<" cannot be loaded."<<inl;
    }
  }
};

/*	loadRoot()
 */
static void
loadRoot()
{
  string igorPathName=openFileDialogue("Choose a root file:");
  if (igorPathName.empty()){
    history<<"Root file loading cancelled"<<inl;
    return;
  }
  FilePath path(igorPathName);
  history<<"Loading from: "<<path.fileName()<<inl;
  history<<"A data folder called "<<path.fileRoot()<<" will be created for this file."<<inl;
  string pathName=path.macPath();
	TFile f(pathName.c_str());
	TIter next(f.GetListOfKeys());
	string msg;
  // Use or create folder
  const string folder=path.fileRoot();
  if (folderExists(folder)){
    cd(folder);
  } else {
    mkdir(folder);
    cd(folder);
  }
  loadRecursively(f,next);
  //
}



/*	XOP1(wavH)
	Carries out operation described at top of file.
	Returns 0 if everything alright or error code otherwise.
 */
static int
LoadRootFile()		
{
	loadRoot();
	return 0;
}

#include "XOPStructureAlignmentTwoByte.h"	// All structures passed to Igor are two-byte aligned.
//
struct XOP1RuntimeParams {					// We receive this structure from Igor when our operation is invoked.	
	// These are postamble fields that Igor sets.
	int calledFromFunction;					// 1 if called from a user function, 0 otherwise.
	int calledFromMacro;					// 1 if called from a macro, 0 otherwise.
};
typedef struct XOP1RuntimeParams XOP1RuntimeParams;
typedef struct XOP1RuntimeParams* XOP1RuntimeParamsPtr;
//
#include "XOPStructureAlignmentReset.h"

/*	DoXOP1Operation(operationInfo)

	DoXOP1Operation is called when the user invokes the XOP1 operation from the
	command line, from a macro or from a user function.
	
	It returns 0 if everything went OK or error code otherwise.
*/
static int
ExecuteXOP1(XOP1RuntimeParamsPtr p)
{
	// Do the operation.
	int result = LoadRootFile();
	return result;
}

/*	XOPMenuItem() added 19/12/2007

XOPMenuItem is called when the XOP's menu item is selected.
*/
static int
XOPMenuItem(void)
{

	int err;
  
	
	err = LoadRootFile();
	return err;
}




static int
RegisterXOP1(void)					// Called at startup to register this operation with Igor.
{
	char* cmdTemplate;
	char* runtimeNumVarList;
	char* runtimeStrVarList;

	// NOTE: If you change this template, you must change the XOP1RuntimeParams structure as well.
	cmdTemplate = "LoadRootFile";
	runtimeNumVarList = "";
	runtimeStrVarList = "";
	return RegisterOperation(cmdTemplate, runtimeNumVarList, runtimeStrVarList, sizeof(XOP1RuntimeParams), reinterpret_cast<void *>(ExecuteXOP1), 0);
}

static int
RegisterOperations(void)		// Register any operations with Igor.
{
	int result;
	
	// Register XOP1 operation.
	if (result = RegisterXOP1())
		return result;
	
	// There are no more operations added by this XOP.
		
	return 0;
}

/*	XOPEntry()

	This is the entry point from the host application to the XOP for all
	messages after the INIT message.
*/
static void
XOPEntry(void)
{	
	long result = 0;
	
	switch (GetXOPMessage()) {
    //added switch cases 19/12/07
    case CLEANUP:
      break;
    case MENUITEM:
      result = XOPMenuItem();
			SetXOPType(TRANSIENT);					// XOP is done now.
			break;
	}
	SetXOPResult(result);
}

/*	main(ioRecHandle)

	This is the initial entry point at which the host application calls XOP.
	The message sent by the host must be INIT.
	
	main does any necessary initialization and then sets the XOPEntry field of the
	ioRecHandle to the address to be called for future messages.
*/
HOST_IMPORT int
main(IORecHandle ioRecHandle)
{
	int result;
	
	XOPInit(ioRecHandle);							// Do standard XOP initialization.

	SetXOPEntry(XOPEntry);							// Set entry point for future calls.

	if (result = RegisterOperations()) {
		SetXOPResult(result);
		return 0;
	}
	
	SetXOPResult(0);
	return 0;
}
