/**
 *  @file FilePath.cpp
 *  XOP1
 *
 *  Created by sroe on 17/11/2007.
 *   2007 CERN.
 *
 */

#include <sstream>
#include <iterator>
#include <algorithm>

#include "FilePath.h"
#include "tokenize.hpp"
#include <cassert>

using namespace std;

string FilePath ::m_delimiter="/";
unsigned int FilePath ::m_maxPathLength=0;
string FilePath ::m_fileSystemRoot="Macintosh HD";


FilePath :: FilePath(const std::string & pathString, const unsigned int maxLength): 
		m_pathType(UNKNOWN){
      m_maxPathLength=0;
      m_delimiter="/";
      m_fileSystemRoot="Macintosh HD";
  string macDelimiter="/";
  string igorDelimiter=":";
  if (maxLength) assert(pathString.size() < maxLength);
  bool macString=(pathString.find(macDelimiter) !=string::npos);
  bool igorString=(pathString.find(igorDelimiter) !=string::npos);
  if (macString==igorString) exit(-1);
  Vec_t::iterator it(m_pathComponents.begin());
  if (macString){
    m_pathType = MAC;
    m_delimiter=macDelimiter;
    m_pathComponents.insert(it++, m_fileSystemRoot);
  }
  if (igorString){
    m_pathType = IGOR;
    m_delimiter= igorDelimiter;
  }
  Vec_t split=tokenize<string>(pathString, m_delimiter);
  Vec_t::const_iterator beg(split.begin());
  Vec_t::const_iterator end(split.end());
  m_pathComponents.insert(it,beg,end);
}

string
FilePath :: igorPath(){
  ostringstream outputStream;
	copy(m_pathComponents.begin(),m_pathComponents.end(),std::ostream_iterator<string>(outputStream,":"));
  string ostring=outputStream.str();
  return ostring.substr(0, (ostring.size()-1));
}

string
FilePath :: macPath(){
  ostringstream outputStream;
	copy(++(m_pathComponents.begin()),m_pathComponents.end(),std::ostream_iterator<string>(outputStream,"/"));
  string ostring=outputStream.str();
  string chomp= ostring.substr(0, (ostring.size()-1));
  return string("/")+chomp;
}

string
FilePath :: fileName(){
  unsigned int last=m_pathComponents.size()-1;
  string fname=m_pathComponents[last];
  return fname;
}

string
FilePath :: fileRoot(){
  string fname=fileName();
  Vec_t split=tokenize<string>(fname,string("."));
  return split[0];
}



