/*
 *  Wave.h
 *  XOP1
 *
 *  Created by sroe on 29/10/2007.
 *  Copyright 2007 CERN. All rights reserved.
 *
 */
#ifndef Wave_hpp
#define Wave_hpp
#include "XOPStandardHeaders.h"
#include "IgorTypeTraits.hpp"

#include <vector>
#include <string>
#include "boost/multi_array.hpp"
#include "dataFolderUtilities.h"


template <typename T>
class Wave{
private:
  T * m_pdata;
  long m_dimensionSizes[MAX_DIMENSIONS+1];
  //unsigned int m_numElements;
  waveHndl m_pwave;
  DataFolderHandle m_pdataFolder;
  T * m_pwaveData;
  Handle m_noteHandle;
  int m_numDimensions;
  bool ok() const{
    return (m_pwave != NULL);
  }

public:
    //1-D constructor from vector<D>
  template <typename D>
    Wave(const std::string &name, const std::vector<D> & dataVector){
    m_numDimensions=1;
    MemClear(m_dimensionSizes, sizeof(m_dimensionSizes));
    //m_numElements=dataVector.size();
    m_dimensionSizes[ROWS] = dataVector.size();
    m_pdataFolder=NULL;
    MDMakeWave(&m_pwave, name.c_str(), m_pdataFolder, m_dimensionSizes, IgorTypeTraits<T>::igorType, m_numDimensions);
    if (ok()){
      int heapState=MoveLockHandle(m_pwave);
      m_pwaveData = (T *)WaveData(m_pwave);
      for(int i(0);i<m_dimensionSizes[ROWS];i++){
        T igorData = (T)(dataVector[i]);	
        *m_pwaveData++ = igorData;
      }
      HSetState(m_pwave, heapState);
      m_noteHandle=NULL;
    }
  }
  // 2-d constructor from boost::multi_array<D,2>
  template <typename D>
    Wave(const std::string &name, boost::multi_array<D, 2> &data2dArray){
      m_numDimensions=2;
      MemClear(m_dimensionSizes, sizeof(m_dimensionSizes));
      m_dimensionSizes[ROWS] = data2dArray.shape()[0];
      m_dimensionSizes[COLUMNS]=data2dArray.shape()[1];
      m_pdataFolder=NULL;
      MDMakeWave(&m_pwave, name.c_str(), m_pdataFolder, m_dimensionSizes, IgorTypeTraits<T>::igorType, m_numDimensions);
      if (ok()){
        int heapState=MoveLockHandle(m_pwave);
        m_pwaveData = (T *)WaveData(m_pwave);
        for ( int c(0);c<m_dimensionSizes[COLUMNS];++c){
          for ( int r(0);r<m_dimensionSizes[ROWS];++r){
            T igorData = (T)(data2dArray[r][c]);
            *m_pwaveData++ = igorData;
          }
        }
        HSetState(m_pwave, heapState);
        m_noteHandle=NULL;
      }
    }
  
	unsigned int size(const int dimension) const{
    if (not ok()) return 0; 
    return m_dimensionSizes[dimension];
  }
  int setWaveNote(const std::string & noteText){
    if (not ok()) return -1;
    Handle h=NewHandle(noteText.length());
    if (h==NULL) return NOMEM;
    PutCStringInHandle(noteText.c_str(),h);
    SetWaveNote(m_pwave,h);
    return 0;
  }
  int setRange(const double lo, const double hi, const int dimension = 0){
    if (dimension >= m_numDimensions) return -1;
    if (not ok()) return -1;
    unsigned int npts = size(dimension);
    double dx=(hi - lo)/npts;
    double nulo=(m_numDimensions==1)?lo:(0.5*dx+lo);
    return MDSetWaveScaling(m_pwave, dimension, &dx, (double *)&nulo);
  }
  
  operator bool () const{
    return ok();
  }
protected:
  
  
};
#endif

