/*
 *  @file buffered_streambuf.hpp
 *  streamBuffer
 *
 *  Created by sroe on 27/10/2007.
 *  Copyright 2007 CERN. All rights reserved.
 *
 */

#ifndef buffered_streambuf_h
#define buffered_streambuf_h
#include <streambuf>
#include "buffer_to_device.h"

class  buffered_streambuf: public std::basic_streambuf<char,std::char_traits<char> >
{
  
public:
  buffered_streambuf();
  ~buffered_streambuf();

  
protected:
  virtual int_type xsputn(const char_type *s, int_type n);
  virtual int_type overflow(int_type c=traits_type::eof());
  virtual int_type sync();
 
private:
  static const int bufSize=200;
  char_type buffer[bufSize];
  int buffer_out();
  
  buffered_streambuf(const buffered_streambuf &);
  buffered_streambuf & operator = (const buffered_streambuf &);
  
};

buffered_streambuf::buffered_streambuf(){
  setp(buffer, buffer + bufSize); 
}

buffered_streambuf::~buffered_streambuf(){
  sync(); 
}

buffered_streambuf::int_type
buffered_streambuf::sync(){
  return buffer_out(); 
}

int
buffered_streambuf::buffer_out(){
  int cnt = pptr() - pbase();
  int retval = buffer_to_device(buffer, cnt);
  pbump(-cnt);
  return retval;
}

buffered_streambuf::int_type
buffered_streambuf::overflow(int_type c){
  if (buffer_out()< 0){
    return traits_type::eof(); 
  } else {
    if (not traits_type::eq_int_type(c, traits_type::eof())){
      return sputc(c); 
    } else {
      return traits_type::not_eof(c);
    }
  }
}

buffered_streambuf::int_type
buffered_streambuf::xsputn(const char_type *s, buffered_streambuf::int_type n){
  if (n<epptr() - pptr()) {
    memcpy(pptr(), s, n * sizeof(char_type));
    pbump(n);
    return n;
  } else {
    for (int_type i=0; i<n; i++){
      if (traits_type::eq_int_type(sputc(s[i]), traits_type::eof())){
        return i; 
      }
    }
    return n;
  }
}

#endif



