/*
 *  load.cpp
 *  XOP1
 *
 *  Created by sroe on 22/11/2007.
 *
 */
#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TProfile.h"
#include "TProfile2D.h"
#include "TKey.h"
#include "Wave.hpp"
#include "simpleArrayUtilities.hpp"
#include <vector>
#include "dataFolderUtilities.h"
//#include <iostream>

#include "load.h"
/** TODO
 *  Factor out common code
 *  Separate tree traversal(root) and wave creation(igor) responsibilities
 **/

using namespace std;
//
static string loadable[]={"TH1F", "TH2F","TH1D", "TH2D", "TH1I","TProfile","TProfile2D"};
enum {th1f, th2f,th1d, th2d, th1i, tprofile1d,tprofile2d};
//
bool
loadFromTFile(TDirectory & f, const string & className, const string & name){
  if (not (contains(loadable, className))) return false;
  int lbl=strIndex(loadable, className);
  switch(lbl){
    //
    case th1f:{
      TH1 *hpx=(TH1F*)f.Get(name.c_str());
      int nbins=hpx->GetNbinsX();
      float xmin = hpx->GetXaxis()->GetXmin();
      float xmax = hpx->GetXaxis()->GetXmax();
      vector<float> contents;
      for(int i=1;i<=nbins;i++){
        contents.push_back(hpx->GetBinContent(i));
      }
      string theTitle=hpx->GetTitle();
      Wave<float> mywave(name, contents);
      if (mywave){
        mywave.setWaveNote(theTitle);
        mywave.setRange(xmin, xmax);
      }
      break;
    }
    //
    case th2f:{
      TH2 *hpxpy=(TH2F*)f.Get(name.c_str());
      int nxbins=hpxpy->GetNbinsX();
      int nybins=hpxpy->GetNbinsY();
      float xmin = hpxpy->GetXaxis()->GetXmin();
      float xmax = hpxpy->GetXaxis()->GetXmax();
      float ymin = hpxpy->GetYaxis()->GetXmin();
      float ymax = hpxpy->GetYaxis()->GetXmax();
      boost::multi_array<float,2> a(boost::extents[nxbins][nybins]);
      for(int x(1);x<=nxbins;++x){
        for(int y(1);y<=nybins;++y){
          a[x-1][y-1]=hpxpy->GetBinContent(x,y);
        }
      }
      string theTitle=hpxpy->GetTitle();
      Wave<float> my2dWave(name, a);
      if (my2dWave){
        my2dWave.setWaveNote(theTitle);
        my2dWave.setRange(xmin, xmax,ROWS);
        my2dWave.setRange(ymin, ymax,COLUMNS);
      }
      break;
    }
      
    case th1d:{
      TH1 *hpx=(TH1*)f.Get(name.c_str());
      int nbins=hpx->GetNbinsX();
      float xmin = hpx->GetXaxis()->GetXmin();
      float xmax = hpx->GetXaxis()->GetXmax();
      vector<double> contents;
      for(int i=1;i<=nbins;i++){
        contents.push_back(hpx->GetBinContent(i));
      }
      string theTitle=hpx->GetTitle();
      Wave<double> mywave(name, contents);
      if (mywave){
        mywave.setWaveNote(theTitle);
        mywave.setRange(xmin, xmax);
      }
      break;
    }
    //
    case th2d:{
      TH2 *hpxpy=(TH2D*)f.Get(name.c_str());
      int nxbins=hpxpy->GetNbinsX();
      int nybins=hpxpy->GetNbinsY();
      float xmin = hpxpy->GetXaxis()->GetXmin();
      float xmax = hpxpy->GetXaxis()->GetXmax();
      float ymin = hpxpy->GetYaxis()->GetXmin();
      float ymax = hpxpy->GetYaxis()->GetXmax();
      boost::multi_array<double,2> a(boost::extents[nxbins][nybins]);
      for(int x(1);x<=nxbins;++x){
        for(int y(1);y<=nybins;++y){
          a[x-1][y-1]=hpxpy->GetBinContent(x,y);
        }
      }
      string theTitle=hpxpy->GetTitle();
      Wave<double> my2dWave(name, a);
      if (my2dWave){
        my2dWave.setWaveNote(theTitle);
        my2dWave.setRange(xmin, xmax,ROWS);
        my2dWave.setRange(ymin, ymax,COLUMNS);        
      }
      break;
    }
    //
    case th1i:{
      TH1 *hpx=(TH1*)f.Get(name.c_str());
      int nbins=hpx->GetNbinsX();
      float xmin = hpx->GetXaxis()->GetXmin();
      float xmax = hpx->GetXaxis()->GetXmax();
      vector<int> contents;
      for(int i=1;i<=nbins;i++){
        contents.push_back(hpx->GetBinContent(i));
      }
      string theTitle=hpx->GetTitle();
      Wave<int> mywave(name, contents);
      if (mywave){
        mywave.setWaveNote(theTitle);
        mywave.setRange(xmin, xmax);
      }
      break;
    }
    //
    case tprofile1d:{
      string profileFolderName=string("tp_")+name;
      string errWaveName=name+"_err";
      string entriesWaveName=name+"_entries";
      mkdir(profileFolderName);
      cd(profileFolderName);
      TProfile *hpx=(TProfile*)f.Get(name.c_str());
      int nbins=hpx->GetNbinsX();
      float xmin = hpx->GetXaxis()->GetXmin();
      float xmax = hpx->GetXaxis()->GetXmax();
      vector<double> contents;
      vector<double> errs;
      vector<unsigned int> n_entries;
      for(int i=1;i<=nbins;i++){
        contents.push_back(hpx->GetBinContent(i));
        errs.push_back(hpx->GetBinError(i));
        n_entries.push_back(hpx->GetBinEntries(i));
      }
      string theTitle=hpx->GetTitle();
      Wave<double> mywave(name, contents);
      Wave<double> errwave(errWaveName, errs);
      Wave<unsigned long> entrieswave(entriesWaveName, n_entries);
      if (mywave){
        mywave.setWaveNote(theTitle);
        mywave.setRange(xmin, xmax);
      }
      //
      if (errwave){
        string errWaveNote=string("Errors for TProfile wave ")+name;
        errwave.setWaveNote(errWaveNote);
      }
      //
      if (entrieswave){
        string entriesWaveNote=string("Entries for TProfile wave ")+name;
        entrieswave.setWaveNote(entriesWaveNote);
      }
      cd("..");
      break;
    }
    //
    //
    case tprofile2d:{
      string profileFolderName=string("tp2d_")+name;
      string errWaveName=name+"_err";
      string entriesWaveName=name+"_entries";
      mkdir(profileFolderName);
      cd(profileFolderName);
      TProfile2D *hpxpy=(TProfile2D*)f.Get(name.c_str());
      int nxbins=hpxpy->GetNbinsX();
      int nybins=hpxpy->GetNbinsY();
      float xmin = hpxpy->GetXaxis()->GetXmin();
      float xmax = hpxpy->GetXaxis()->GetXmax();
      float ymin = hpxpy->GetYaxis()->GetXmin();
      float ymax = hpxpy->GetYaxis()->GetXmax();
      boost::multi_array<double,2> a(boost::extents[nxbins][nybins]);
      boost::multi_array<double,2> e(boost::extents[nxbins][nybins]);
      boost::multi_array<unsigned int,2> n(boost::extents[nxbins][nybins]);
      for(int x(1);x<=nxbins;++x){
        for(int y(1);y<=nybins;++y){
          a[x-1][y-1]=hpxpy->GetBinContent(x,y);
          e[x-1][y-1]=hpxpy->GetBinError(x,y);
          int b=hpxpy->GetBin(x,y);
          n[x-1][y-1]=hpxpy->GetBinEntries(b);
        }
      }
      string theTitle=hpxpy->GetTitle();
      Wave<double> my2dWave(name, a);
      Wave<double> errwave(errWaveName, e);
      Wave<unsigned long> entrieswave(entriesWaveName, n);
      if (my2dWave){
        my2dWave.setWaveNote(theTitle);
        my2dWave.setRange(xmin, xmax,ROWS);
        my2dWave.setRange(ymin, ymax,COLUMNS);        
      }
      //
      if (errwave){
        string errWaveNote=string("Errors for TProfile2D wave ")+name;
        errwave.setWaveNote(errWaveNote);
      }
      
      if (entrieswave){
        string entriesWaveNote=string("Entries for TProfile2D wave ")+name;
        entrieswave.setWaveNote(entriesWaveNote);
      }
      
      cd("..");
      break;
    }
      
  }
  //
  //case tprofile1d:
    
  return true;
};

void
traverseFileTree(TDirectory &f){
  TIter next(f.GetListOfKeys());
	TKey * key;
  while((key = (TKey*) next())){
    string className=key->GetClassName();
    string name=key->GetName();
		//history<< name <<":"<<className;
    if (className.find("TDirectory") not_eq string::npos){
      TDirectory * thisDirectory=(TDirectory *) key->ReadObj();
      mkdir(name);
      cd(name);
      traverseFileTree(*thisDirectory);
      cd("..");
    } else {
      loadFromTFile(f, className,name);
    }
  }
};


